@@ -1,5 +1,4 @@
-#v1.4.0
-#26.12.15
+#11.08.16
 
 
 
@@ -36,7 +35,9 @@ setMethod(f = "initialize",
 				not_delete_back=NULL,
                                off=NULL,
                                on=NULL,
-                               simple=FALSE,                           
+                               simple=FALSE,  
+                                del_pen_hin=c(),
+				del_pen_ruck=c(),                          
                                reverse_hin=c(),
                                reverse_hin_penalty=c(),
                                reverse_back=c(),
@@ -126,15 +127,8 @@ setMethod(f = "initialize",
 		clb=c(rep(0,length(reverse_hin)),rep(0,length(reverse_back)),rep(0,2*num_additional_reactions),rep(0,2*nc),rep(0,length(additional_biomass_metabolites)),rep(0,length(remove_biomass_metabolites)),rep(0,length(variable_lower_bound)),vec_vlb_max)
 				
 		ctype=c(rep("B",length(reverse_hin)),rep("B",length(reverse_back)),rep("B",2*num_additional_reactions),rep("B",2*nc),rep("B",length(additional_biomass_metabolites)),rep("B",length(remove_biomass_metabolites)),rep("B",length(variable_lower_bound)),rep("C",length(variable_lower_bound)))
-		del_pen=c(rep(1,length(1:(add_start-1))),(rep(0,length(add_start:nc))))
 		
-		
-		if(num_additional_reactions==0)
-		{
-			del_pen=c(rep(1,length(1:(add_start-1))),rep(0,length(additional_biomass_metabolites)))
-		}
-		
-		objectives=c(reverse_hin_penalty,reverse_back_penalty,penalties_hin,penalties_ruck,del_pen,del_pen,additional_biomass_metabolites_pen,remove_biomass_metabolites_pen,vec_vlb_pen,rep(0,length(vec_vlb_pen)))
+		objectives=c(reverse_hin_penalty,reverse_back_penalty,penalties_hin,penalties_ruck,del_pen_hin,del_pen_ruck,additional_biomass_metabolites_pen,remove_biomass_metabolites_pen,vec_vlb_pen,rep(0,length(vec_vlb_pen)))
 		
 		
 		length_objectives=length(objectives)
